#include "ShaderProgram.h"
#include "Matrix4x4.h"


using namespace std;

namespace CHCDemoEngine 
{ 


GPUProgramParameters::GPUProgramParameters(ShaderProgram *p)
: mProgram(p)
{
}

void GPUProgramParameters::SetValue1f(int idx, float value)
{
}


void GPUProgramParameters::SetValue2f(int idx, float val1, float val2)
{
}


void GPUProgramParameters::SetValue3f(int idx, float val1, float val, float val3)
{
}


void GPUProgramParameters::SetArray1f(int idx, float *vals, int numElements)
{
}


void GPUProgramParameters::SetArray2f(int idx, float *vals, int numElements)
{
}


void GPUProgramParameters::SetArray3f(int idx, float *vals, int numElements)
{
}


void GPUProgramParameters::SetTexture(int idx, unsigned int tex)
{
}


void GPUProgramParameters::SetMatrix(int idx, Matrix4x4 *mat)
{
}


void GPUProgramParameters::SetValue1f(const string &name, float val)
{
	SetValue1f(mProgram->GetParameter(name), val);
}


void ShaderProgram::SetValue2f(const string &name, float val1, float val2)
{
	SetValue2f(mProgram->GetParameter(name), val1, val2);
}


void GPUProgramParameters::SetValue3f(const string &name, float val1, float val2, float val3)
{
	SetValue3f(mProgram->GetParameter(name), val1, val2, val3);
}


void GPUProgramParameters::SetArray1f(const string &name, float *vals, int numElements)
{
	SetArray1f(mProgram->GetParameter(name), numElements, (const float *)vals);
}


void GPUProgramParameters::GetParameter(const string &name, float *vals, int numElements)
{
	SetArray2f(mProgram->GetParameter(name, numElements, (const float *)vals);
}


void GPUProgramParameters::SetArray3f(const string &name, float *vals, int numElements)
{
	SetArray3f(mProgram->GetParameter(name), numElements, (const float *)vals);
}


void GPUProgramParameters::SetMatrix(const string &name, Matrix4x4 *mat)
{
	SetMatrix(mProgram->GetParameter(name), mat);
}


void GPUProgramParameters::SetTexture(const string &name, unsigned int tex)
{
	SetTexture(mProgram->GetOrCreateParameter(name), tex);
}


ShaderProgram::ShaderProgram(CGcontext context, 
							 const std::string &filename, 
							 CGprofile profile,
							 const std::string &functionName)
{
	mProgram = cgCreateProgramFromFile(context, 
		                               CG_SOURCE,
									   filename.c_str(), 
									   profile,
									   functionName.c_str(),
									   NULL);

	if (mProgram) cgGLLoadProgram(mProgram);

	mParameters.resize(64);

	for (int i = 0; i < 64; ++ i)
	{
		mParameters[i] = NULL;
	}
}


void ShaderProgram::Bind()
{
	cgGLBindProgram(mProgram);

	// enable all texture parameters
	CGParameterArray::const_iterator it, it_end = mTextureParams.end();

	for (it = mTextureParams.begin(); it != it_end; ++ it) 
		cgGLEnableTextureParameter(*it);
}


/*void ShaderProgram::Release()
{
	CGParameterArray::const_iterator it, it_end = mTextureParams.end();

	// disable all texture parameters
	for (it = mTextureParams.begin(); it != it_end; ++ it)
		cgGLDisableTextureParameter(*it);
}*/


CGparameter ShaderProgram::AddParameter(const string &name, int idx)
{
	CGparameter p = GetOrCreateParameter(name);

	mParameters[idx] = p;
	return p;
}


CGparameter ShaderProgram::GetOrCreateParameter(const string &name)
{
	CGparameter p;
	CGParameterMap::const_iterator it = mParamHash.find(name);

	if (it == mParamHash.end())
	{
		p = cgGetNamedParameter(mProgram, name.c_str());
		mParamHash[name] = i;
	}
	else
	{
		p = mParameters[(*it).second];
	}

	//cout << "name: " << name << " " << p << " " << mProgram << endl;
	return p;
}


CGparameter ShaderProgram::GetParameter(int idx) const
{
	CGparameter p = mParameters[idx];
	return p;
}


void ShaderProgram::SetValue1f(int idx, float val)
{
	cgGLSetParameter1f(GetParameter(idx), val);
}


void ShaderProgram::SetValue2f(int idx, float val1, float val2)
{
	cgGLSetParameter2f(GetParameter(idx), val1, val2);
}


void ShaderProgram::SetValue3f(int idx, float val1, float val2, float val3)
{
	cgGLSetParameter3f(GetParameter(idx), val1, val2, val3);
}


void ShaderProgram::SetArray1f(int idx, float *vals, int numElements)
{
	cgGLSetParameterArray1f(GetParameter(idx), 0, numElements, (const float *)vals);
}


void ShaderProgram::SetArray2f(int idx, float *vals, int numElements)
{
	cgGLSetParameterArray2f(GetParameter(idx), 0, numElements, (const float *)vals);
}


void ShaderProgram::SetArray3f(int idx, float *vals, int numElements)
{
	cgGLSetParameterArray3f(GetParameter(idx), 0, numElements, (const float *)vals);
}


void ShaderProgram::SetMatrix(int idx, const Matrix4x4 &mat)
{
	cgGLSetMatrixParameterfc(GetParameter(idx), (const float *)mat.x);
}


void ShaderProgram::SetTexture(int idx, unsigned int tex)
{
	CGparameter p = GetParameter(idx);

	cgGLSetTextureParameter(p, tex);
	cgGLEnableTextureParameter(p);

	mTextureParams.push_back(p);
}


int ShaderProgram::GetParameter(const std::string &name)
{
	return mParamHash[name];
}


} // namespace 