#ifndef _SHADERPROGRAM_H__
#define _SHADERPROGRAM_H__

#include "common.h"
#include "glInterface.h"

#include <Cg/cg.h>
#include <Cg/cgGL.h>
#include <string>
#include <map>


namespace CHCDemoEngine 
{ 

class FrameBufferObject;
class Vector3;
class Camera;
class Matrix4x4;
class ShadowMap;
class DirectionalLight;
class ShaderProgram;


typedef std::map<std::string, int> CGParameterMap;
typedef std::vector<CGparameter> CGParameterArray;

struct ParameterPair1f
{
	unsigned int mParameterIdx;
	float mValue;
};

class GPUProgramParameters
{
	void SetValue1f(int idx, float value);
	void SetValue2f(int idx, float val1, float val2);
	void SetValue3f(int idx, float val1, float val, float val3);

	void SetArray1f(int idx, float *vals, int numElements);
	void SetArray2f(int idx, float *vals, int numElements);
	void SetArray3f(int idx, float *vals, int numElements);

	/** Sets the texture parameter.
	*/
	void SetTexture(int idx, unsigned int tex);
	/** Sets the matrix parameter.
	*/
	void SetMatrix(int idx, Matrix4x4 *mat);

	void SetValue1f(const std::string &name, float value);
	void SetValue2f(const std::string &name, float val1, float val2);
	void SetValue3f(const std::string &name, float val1, float val, float val3);

	void SetArray1f(const std::string &name, float *vals, int numElements);
	void SetArray2f(const std::string &name, float *vals, int numElements);
	void SetArray3f(const std::string &name, float *vals, int numElements);
	/** Sets the texture parameter.
	*/
	void SetTexture(const std::string &name, unsigned int tex);
	/** Sets the matrix parameter.
	*/
	void SetMatrix(const std::string &name, Matrix4x4 *mat);

	struct ParameterFloat
	{
		int mIndex;
		float mValues[4];
		float mNumComponents;
	};

	struct ParameterValueInt
	{
		int mIndex;
		float mValue;
	};

	struct ParameterValueMatrix
	{
		int mIndex;
		Matrix4x4 *mValue;
	};

	struct ParameterValueArray
	{
		int mIndex;

		float *mValues;
		int mNumEntries;

		float mNumComponents;
	};


	GPUProgramParameters(ShaderProgram *p);


protected:

	ShaderProgram *mProgram;

	std::vector<ParameterFloat> mFloats;

	std::vector<ParameterValueInt> mTextures;
	std::vector<ParameterValueMatrix> mMatrices;
};



class ShaderProgram
{
	friend class GPUProgramParameters;

public:

	ShaderProgram(CGprogram program): mProgram(program) {}

	ShaderProgram(CGcontext context, 
		          const std::string &filename, 
				  CGprofile profile,
				  const std::string &functionName);

	~ShaderProgram() { if (mProgram != NULL) cgDestroyProgram(mProgram); }

	void SetValue1f(int idx, float value);
	void SetValue2f(int idx, float val1, float val2);
	void SetValue3f(int idx, float val1, float val, float val3);

	void SetArray1f(int idx, float *vals, int numElements);
	void SetArray2f(int idx, float *vals, int numElements);
	void SetArray3f(int idx, float *vals, int numElements);
	void SetMatrix(int idx, const Matrix4x4 &mat);

	void SetTexture(int idx, unsigned int tex);

	CGparameter AddParameter(const std::string &name, int idx);

	/** Enable / disable a texture parameter.
	*/
	//void EnableTexture(const std::string &name);
	//void DisableTexture(const std::string &name);

	/** Binds the program.
	*/
	void Bind();
	/** Returns true if this program is valid.
	*/
	inline bool IsValid() const { return mProgram != NULL; }

	inline int GetParameter(const std::string &name);


protected:

	inline CGparameter GetParameter(int idx);

	CGparameter GetOrCreateParameter(const std::string &name);


	///////////////

	CGParameterMap mParamHash;
	
	CGprogram mProgram;

	CGParameterArray mTextureParams;
	CGParameterArray mParameters;
};


typedef  std::vector<ShaderProgram *> ShaderContainer;

} // namespace 

#endif // _ShaderProgram_H__